/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.hudson.bears;

import org.apache.commons.io.IOUtils;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.MalformedURLException;

/**
 * The bear updater
 */
public class BearUpdater {
  public static void main( String[] args ) throws Exception {
    if ( args.length == 0 ) {
      throw new IllegalArgumentException( "Need two parameters: <URL to Hudson> <Host/IP of plug connector>" );
    }

    String hudsonUrl = args[0].trim();
    String hostPlugConnector = args[1].trim();

    new BearUpdater( hudsonUrl, hostPlugConnector ).run();
  }

  @NonNls
  @NotNull
  private final StatusUrls statusUrls;
  @NotNull
  private final PlugConnector plugConnector;

  private volatile boolean shuttingDown;

  public BearUpdater( @NotNull @NonNls String hudsonUrl, @NotNull @NonNls String hostPlugConnector ) throws MalformedURLException {
    statusUrls = new StatusUrls( hudsonUrl );
    plugConnector = new PlugConnector( hostPlugConnector );

    System.out.println( "Starting Bear Updater..." );
    System.out.println( "\tConnecting to Hudson @ <" + hudsonUrl + ">" );
    System.out.println( "\tConnecting to Plug Connector @ <" + hostPlugConnector + ">" );
  }

  private void run() throws IOException {
    Runtime.getRuntime().addShutdownHook( new Thread( new Runnable() {
      public void run() {
        shuttingDown = true;
        System.out.println( "Disabling all" );
        try {
          plugConnector.disableAll();
        } catch ( IOException e ) {
          e.printStackTrace();
        }
      }
    } ) );

    while ( true ) {
      if ( Thread.currentThread().isInterrupted() ) {
        return;
      }

      ProjectsState projectsState = parseProjectsState();
      if ( shuttingDown ) {
        return;
      }
      switchBears( projectsState );

      try {
        Thread.sleep( 1000 * 60 );
      } catch ( InterruptedException ignore ) {
      }
    }
  }

  private void switchBears( @NotNull ProjectsState criticalState ) throws IOException {
    PlugState[] plugStates = {PlugState.OFF, PlugState.OFF, PlugState.OFF, PlugState.OFF};

    if ( criticalState.hasBuildingProject() ) {
      plugStates[0] = PlugState.ON;
    }

    if ( criticalState.hasFailedProject() ) {
      plugStates[3] = PlugState.ON;
    } else {
      if ( criticalState.hasInstableProject() ) {
        plugStates[2] = PlugState.ON;
      } else {
        if ( criticalState.hasStableProject() ) {
          plugStates[1] = PlugState.ON;
        }
      }
    }
    plugConnector.switchPlugs( plugStates );
  }

  @NotNull
  private ProjectsState parseProjectsState() throws IOException {
    String statusXml = IOUtils.toString( statusUrls.getCritical().openStream() );

    ProjectsState projectsState = new ProjectsState();

    for ( ProjectState state : ProjectState.values() ) {
      if ( statusXml.contains( state.getHudsonColor() + "</color>" ) ) {
        projectsState.enable( state );
      }
    }
    return projectsState;
  }
}
